import numpy as np
import matplotlib.pyplot as plt
import sympy as sym
import math
import os
import subprocess
import fileinput
import re
import matlab.engine
import sys
from ClassifyMin import *
from mpl_toolkits.mplot3d import Axes3D
import matplotlib.cm as cm
from vtk.util import numpy_support
from pyevtk.hl import gridToVTK
# print(sys.executable)
# from subprocess import Popen, PIPE

# Not Needed:
# def ndm(*args):
#     return [x[(None,)*i+(slice(None),)+(None,)*(len(args)-i-1)] for i, x in enumerate(args)]

# ---------------------------------------------- Main ---------------------s
# --- Input Parameters ----
mu_1 = 1.0
rho_1 = 1.0
alpha = 9.0
beta = 2.0
theta = 1.0/8.0
# print('Type of theta', type(theta))

# ------- Options --------
# print_Cases = True
# print_Output = True

# make_3D_plot = True
# make_2D_plot = False
# make_3D_PhaseDiagram = True
# make_2D_PhaseDiagram = False

make_3D_plot = False
make_2D_plot = True
make_3D_PhaseDiagram = False
make_2D_PhaseDiagram = True


# --- Define q1, q2 , mu_gamma, q12 ---
# 1. read from Cell-Output
# 2. define values from analytic formulas (expect for mu_gamma)
q1 = harmonicMean(mu_1, beta, theta)
q2 = arithmeticMean(mu_1, beta, theta)

# TEST / TODO read from Cell-Output
q12 = 0.0  # (analytical example)

# TODO read from Cell-Output
# --- Set mu_gamma  to value
mu_gamma = q1   # TODO read from Cell-Output


b1 = prestrain_b1(rho_1, beta, alpha, theta)
b2 = prestrain_b2(rho_1, beta, alpha, theta)

print('---- Input parameters: -----')
print('mu_1: ', mu_1)
print('rho_1: ', rho_1)
print('alpha: ', alpha)
print('beta: ', beta)
print('theta: ', theta)
print("q1: ", q1)
print("q2: ", q2)
print("mu_gamma: ", mu_gamma)
print("q12: ", q12)
print("b1: ", b1)
print("b2: ", b2)
print('----------------------------')
# print("machine epsilon", sys.float_info.epsilon)


# G, angle, type, kappa = classifyMin(q1, q2, mu_gamma, q12,  b1, b2, print_Cases, print_Output)

# Test = f(1,2 ,q1,q2,mu_gamma,q12,b1,b2)
# print("Test", Test)

# ---------------------- MAKE PLOT / Write to VTK------------------------------------------------------------------------------

SamplePoints_3D = 100 # Number of sample points in each direction
SamplePoints_2D = 800 # Number of sample points in each direction

if make_3D_PhaseDiagram:
    alphas_ = np.linspace(-20, 20, SamplePoints_3D)
    betas_  = np.linspace(0.01,40.01,SamplePoints_3D)
    thetas_ = np.linspace(0.01,0.99,SamplePoints_3D)
    # print('type of alphas', type(alphas_))
    # print('Test:', type(np.array([mu_gamma])) )
    alphas, betas, thetas = np.meshgrid(alphas_, betas_, thetas_, indexing='ij')
    classifyMin_geoVec = np.vectorize(classifyMin_geo)
    G, angles, Types, curvature = classifyMin_geoVec(alphas,betas,thetas,mu_gamma,q12, mu_1, rho_1)
    # print('size of G:', G.shape)
    # print('G:', G)
    # Out = classifyMin_geoVec(alphas,betas,thetas)
    # T = Out[2]
    # --- Write to VTK
    VTKOutputName = "./PhaseDiagram3D"
    gridToVTK(VTKOutputName , alphas, betas, thetas, pointData = {'Type': Types, 'angles': angles, 'curvature': curvature} )
    print('Written to VTK-File:', VTKOutputName )

if make_2D_PhaseDiagram:
    alphas_ = np.linspace(-20, 20, SamplePoints_2D)
    # betas_  = np.linspace(0.01,40.01,1)
    #fix to one value:
    betas_ = 2.0;
    thetas_ = np.linspace(0.01,0.99,SamplePoints_2D)
    # print('type of alphas', type(alphas_))
    # print('Test:', type(np.array([mu_gamma])) )
    alphas, betas, thetas = np.meshgrid(alphas_, betas_, thetas_, indexing='ij')
    classifyMin_geoVec = np.vectorize(classifyMin_geo)
    G, angles, Types, curvature = classifyMin_geoVec(alphas,betas,thetas,mu_gamma,q12, mu_1, rho_1)
    # print('size of G:', G.shape)
    # print('G:', G)
    # Out = classifyMin_geoVec(alphas,betas,thetas)
    # T = Out[2]
    # --- Write to VTK
    VTKOutputName = "./PhaseDiagram2D"
    gridToVTK(VTKOutputName , alphas, betas, thetas, pointData = {'Type': Types, 'angles': angles, 'curvature': curvature} )
    print('Written to VTK-File:', VTKOutputName )


# --- Make 3D Scatter plot
if(make_3D_plot or make_2D_plot):
    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')
    colors = cm.plasma(Types)
    if make_2D_plot: pnt3d=ax.scatter(alphas,thetas,c=Types.flat)
    # if make_2D_plot: plt.scatter(alphas,thetas,c=Types.flat)
    if make_3D_plot: pnt3d=ax.scatter(alphas,betas,thetas,c=Types.flat)
    # cbar=plt.colorbar(pnt3d)
    # cbar.set_label("Values (units)")
    ax.set_xlabel('alpha')
    ax.set_ylabel('beta')
    if make_3D_plot: ax.set_zlabel('theta')
    plt.show()
    # plt.savefig('common_labels.png', dpi=300)
    # print('T:', T)
    # print('Type 1 occured here:', np.where(T == 1))
    # print('Type 2 occured here:', np.where(T == 2))


# ALTERNATIVE
# colors = ("red", "green", "blue")
# groups = ("Type 1", "Type2", "Type3")
#
# # Create plot
# fig = plt.figure()
# ax = fig.add_subplot(1, 1, 1)
#
# for data, color, group in zip(Types, colors, groups):
#     # x, y = data
#     ax.scatter(alphas, thetas, alpha=0.8, c=color, edgecolors='none', label=group)
#
# plt.title('Matplot scatter plot')
# plt.legend(loc=2)
# plt.show()
